import os
import sys
import pandas as pd

def merge_csv_files(output_csv_path, dataset_name, k, clap_filter, dpo, supcon, filteredout):
    # List to store DataFrames
    dataframes = []
    dataframes_synthetic = []
    
    # Iterate over the range 0 to K to read the CSV files
    for num_process in range(k):
        # extra variable for syntethic CSV names
        file_name_synthetic = None
        if dpo == "True":
            print("Merging DPO CSVs.")
            file_name = f"{output_csv_path}{dataset_name}_dpo_{num_process}.csv"
        elif supcon == "True":
            print("Merging Supervised Contrastive CSVs.")
            file_name = f"{output_csv_path}{dataset_name}_supcon_{num_process}.csv"
        elif filteredout == "True":
            print("Merging Filtered Out CSVs.")
            file_name = f"{output_csv_path}{dataset_name}_filteredout_{num_process}.csv"
        else:
            if clap_filter == "True":
                print("Merging Filtered CSVs.")
                file_name = f"{output_csv_path}{dataset_name}_merged_filtered_{num_process}.csv"
                print("Merging Synthetic CSVs.")
                file_name_synthetic = f"{output_csv_path}{dataset_name}_synthetic_{num_process}.csv"
            else:
                print("Merging Normal CSVs.")
                file_name = f"{output_csv_path}{dataset_name}_merged_{num_process}.csv"

        if os.path.exists(file_name):
            df = pd.read_csv(file_name)
            dataframes.append(df)
            # extra step to merge synthetic in final stages
            if file_name_synthetic is not None:
                df = pd.read_csv(file_name_synthetic)
                dataframes_synthetic.append(df)
        else:
            print(f"File {file_name} does not exist. Skipping.")
    
    # Concatenate all DataFrames
    merged_df = pd.concat(dataframes, ignore_index=True)
    # extra for synthetic
    merged_df_synthetic = None
    if len(dataframes_synthetic) > 0:
        merged_df_synthetic = pd.concat(dataframes_synthetic, ignore_index=True)
    
    # Save the merged DataFrame to a new CSV file
    if dpo == "True":
        output_file = f"{output_csv_path}{dataset_name}_dpo_merged.csv"
    elif filteredout == "True":
        output_file = f"{output_csv_path}{dataset_name}_filteredout_merged.csv"
    elif supcon == "True":
        grouped_df = merged_df.groupby('path_old').agg({
            'path_new': lambda x: list(x),
            'label': 'first',
            'captions': 'first'
        }).reset_index()
        output_file = f"{output_csv_path}{dataset_name}_supcon_merged.csv"
        merged_df = grouped_df
    else:
        output_file = f"{output_csv_path}{dataset_name}_merged.csv"
        output_file_synthetic = f"{output_csv_path}{dataset_name}_merged_synthetic.csv"
    merged_df.to_csv(output_file, index=False)
    print(f"Merged file saved as {output_file}")
    if merged_df_synthetic is not None:
        merged_df_synthetic.to_csv(output_file_synthetic, index=False)
        print(f"Merged file saved as {output_file_synthetic}")


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description='Create JSON for label mapping')
    parser.add_argument('--output_csv_path', type=str, help='Path to the input CSV', required=True)
    parser.add_argument('--dataset_name', type=str, help='Name of the dataset', required=True)
    parser.add_argument('--num', type=int, help='Path to the input CSV', required=True)
    parser.add_argument('--clap_filter', type=str, help='If clap filter was applied', required=True)
    parser.add_argument('--dpo', type=str, help='If dpo csvs need to be merged', required=False, default="False")
    parser.add_argument('--supcon', type=str, help='If supcon csvs need to be merged', required=False, default="False")
    parser.add_argument('--filteredout', type=str, help='If filteredout csvs need to be merged', required=False, default="False")

    args = parser.parse_args()

    # Call the function to merge CSV files
    merge_csv_files(args.output_csv_path, args.dataset_name, args.num, args.clap_filter, args.dpo, args.supcon, args.filteredout)